#!/usr/bin/python

#
# Copyright (C) 2008 by OpenMoko, Inc.
# Written by Marek Lindner <marek@openmoko.com>
# All Rights Reserved
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import os
import sys
import socket
import types
from threading import Thread
#import profile


UMASK = 0
WORKDIR = "/"
MAXFD = 1024
UNIX_PATH = "/tmp/app-launcherd.sock"


# The standard I/O file descriptors are redirected to /dev/null by default.
if (hasattr(os, "devnull")):
	REDIRECT_TO = os.devnull
else:
	REDIRECT_TO = "/dev/null"

def createDaemon():
	try:
		pid = os.fork()
	except OSError, e:
		raise Exception, "%s [%d]" % (e.strerror, e.errno)

	## first child
	if (pid == 0):
		## make it session leader
		os.setsid()

		try:
			pid = os.fork()
		except OSError, e:
			raise Exception, "%s [%d]" % (e.strerror, e.errno)

		## second child
		if (pid == 0):
			os.chdir(WORKDIR)
			os.umask(UMASK)
		else:
			## second parent
			os._exit(0)

	else:
		## first parent
		os._exit(0)

	## daemon process
	# Iterate through and close all file descriptors.
	for fd in range(0, MAXFD):
		try:
			os.close(fd)
		## ignore error (unopened fds)
		except OSError:
			pass

	## redirect stdin, stdout and stderr to /dev/null
	os.open(REDIRECT_TO, os.O_RDWR)
	os.dup2(0, 1)
	os.dup2(0, 2)

	return(0)

if __name__ == "__main__":

	myargv = sys.argv[1:]

	if (len(myargv) > 0) and (myargv[0] == "-f"):
		myargv.pop(0)
	else:
		createDaemon()

	## for the applications
	import etk
	import edje
	import ecore.evas

	first_run = 1
	running_threads = {}

	if os.path.exists(UNIX_PATH):
		os.remove(UNIX_PATH)

	server = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
	server.bind(UNIX_PATH)
	#server.settimeout(10.0)

	while True:
		if (first_run) and (len(myargv) > 0):
			client_data = myargv[0]
		else:
			try:
				client_data = server.recv(1024)
			except socket.timeout:
				continue
			except socket.error, msg:
				print "Exiting daemon: " + msg[1]
				break

		first_run = 0

		## try to find the module
		try:
			client_module = sys.modules[client_data]
			if not isinstance(client_module, types.ModuleType):
				raise KeyError
		except KeyError:
			try:
				client_module = __import__(client_data, globals(), locals(), [''])
			except:
				print "Given module not found: " + client_data
				continue

			sys.modules[client_data] = client_module

		## try to find the main class
		try:
			client_class = getattr(client_module, client_data)
		except:
			continue


		## make the window the top window if the module is already running
		if (client_data in running_threads) and running_threads[client_data].isAlive():
			running_threads[client_data].wake_up()
		else:
			## load and start the module
			running_threads[client_data] = client_class(etk, edje, ecore)
			running_threads[client_data].setName(client_data)
			running_threads[client_data].start()

		#running_threads[client_data] = client_class(etk, edje, ecore)
		#profile.run('running_threads[client_data].run()', 'out_file.prof')

	## close all threads
	for mythread in running_threads:
		if (running_threads[mythread].isAlive()):
			running_threads[mythread].stop()
			running_threads[mythread].join()

	server.close()

