#!/usr/bin/python

#
# Copyright (C) 2008 by OpenMoko, Inc.
# Written by Marek Lindner <marek@openmoko.com>
# All Rights Reserved
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import sys
import dbus
import e_dbus
from threading import Thread

from expos import gsm
from expos import bt
from expos import gps
from expos import brightness
from expos import suspend
from expos import time
from expos import sysinfo
from expos import wifi
from expos import profile


### TODO ###

## modules ##
# GSM -> needs Qtopia dbus call (holger)
# WiFi -> needs network manager (raster)
# Bluetooth -> DONE
# GPS -> DONE
# Backlight -> DONE
# Suspend -> DONE
# Profile -> DONE
# Time -> DONE
# System Info -> DONE
##

## misc ##
# load backlight settings on boot
##


class exposure(Thread):
	def __init__(self, etk, edje, ecore, name="exposure"):
		Thread.__init__(self)
		self.name = name
		self.etk = etk
		self.edje = edje
		self.ecore = ecore

	def run(self):

		self.mainloop = e_dbus.DBusEcoreMainLoop()
		self.session_bus = dbus.SessionBus(mainloop=self.mainloop)
		#self.session_bus = dbus.SessionBus()

		### load and initialize modules ###

		self.mygsm = gsm()
		self.mybt = bt()
		self.mygps = gps()
		self.mywifi = wifi(self.etk, dbus)
		self.mybrightness = brightness()
		self.mysuspend = suspend(dbus, self.session_bus)
		self.mytime = time()
		self.mysysinfo = sysinfo(self.etk)
		self.myprofile = profile(dbus, self.session_bus)

		self.loaded_modules = [self.mygsm, self.mywifi, self.mybt, self.mygps, self.mybrightness, self.myprofile, self.mysuspend, self.mytime, self.mysysinfo]


		### window init ###

		if self.ecore.evas.engine_type_supported_get("software_x11_16"):
			self.window = self.ecore.evas.SoftwareX11_16(w=480, h=575)
		else:
			self.window = self.ecore.evas.SoftwareX11(w=480, h=575)

		self.window.callback_delete_request = self.window_delete_request
		self.window.title = "Exposure"

		self.edje_file = "/usr/share/exposure/exposure.edj"
		self.edj = self.edje.Edje(self.window.evas, file=self.edje_file, group="exposure")
		self.edj.size = self.window.evas.size
		self.window.data["edje"] = self.edj
		self.edje.frametime_set(1.0/30)

		self.edj.signal_callback_add("mouse,up,1", "arrow-event", self.arrow_clicked)
		self.edj.show()

		self.mysysinfo.edj_object = self.edj
		self.mysysinfo.edje_file = self.edje_file
		self.mysysinfo.edje = self.edje.Edje
		self.mysysinfo.canvas = self.window.evas

		self.mywifi.edj_object = self.edj
		self.mywifi.edje_file = self.edje_file
		self.mywifi.edje = self.edje.Edje
		self.mywifi.canvas = self.window.evas


		### gui setup ###

		self.vbox = self.etk.VBox()
		self.scrolled_view = self.etk.ScrolledView()
		self.scrolled_view.add_with_viewport(self.vbox)
		## hide scrollbars
		self.scrolled_view.policy_set(2, 2)
		## make it dragable
		self.scrolled_view.dragable_set(1)

		for index, item in enumerate(self.loaded_modules):
			item.timer_add = self.ecore.timer_add
			item.canvas_obj = self.etk.Canvas()
			item.edje_obj = self.edje.Edje(self.window.evas, file=self.edje_file, group="list_item")
			item.canvas_obj.object_add(item.edje_obj)
			item.edje_obj.signal_callback_add("after_animate_click", "list_item", item.after_animate_click)
			item.edje_obj.part_text_set("item-label", item.name)
			item.init_label()
			self.vbox.append(item.canvas_obj, self.etk.VBox.START, self.etk.VBox.NONE, 0)
			item.edje_obj.show()

		## add empty item so that the last item can scroll into the visible screen
		self.last_item_canvas_obj = self.etk.Canvas()
		self.last_item_edje_obj = self.edje.Edje(self.window.evas, file=self.edje_file, group="list_item_unclickable")
		self.last_item_canvas_obj.object_add(self.last_item_edje_obj)
		self.vbox.append(self.last_item_canvas_obj, self.etk.VBox.START, self.etk.VBox.NONE, 0)
		self.last_item_edje_obj.show()

		self.mainmenu_win = self.etk.Embed(self.window.evas)
		self.mainmenu_win.add(self.scrolled_view)
		self.mainmenu_win.show_all()
		self.edj.part_swallow("main-menu", self.mainmenu_win.object)

		self.sysinfo_win = self.etk.Embed(self.window.evas)
		self.sysinfo_win.add(self.mysysinfo.scrolled_view)
		self.sysinfo_win.show_all()
		self.edj.part_swallow("sysinfo", self.sysinfo_win.object)

		self.wifi_win = self.etk.Embed(self.window.evas)
		self.wifi_win.add(self.mywifi.scrolled_view)
		self.wifi_win.show_all()
		self.edj.part_swallow("wifi", self.wifi_win.object)

		self.window.show()
		self.ecore.main_loop_begin()

		self.cleanup()

	def stop(self):
		self.ecore.main_loop_quit()
		self.cleanup()

	def cleanup(self):
		#self.edj.part_unswallow(self.sysinfo_win.object)
		#self.edj.part_unswallow(self.wifi_win.object)
		#self.edj.part_unswallow(self.mainmenu_win.object)
		#self.edj.delete()
		self.window.hide()
		self.ecore.main_loop_iterate()

	def window_delete_request(self, evas_obj):
		self.ecore.main_loop_quit()

	def arrow_clicked(self, edje_obj, emission, source):
		self.edj.part_text_set("settings-text", "Settings")

	def wake_up(self):
		print dir(self.window)
		#self.window.ecore_evas_raise()
		self.window.hide()
		self.window.show()
		#self.window.raise_()
		#self.window.focus_set(1)


if __name__ == "__main__":
	print "This is a module which has to be launched via the app-launcherd.py:"
	print "python app-launcherd.py exposure"

